@@ -60,6 +60,7 @@ has log_encoding        => undef;      #utf8|gbk|...
 has log_head            => "[wxc][$$]";
 has log_console         => 1;
 has disable_color       => 0;
+has max_clients         => 100;
 
 has version             => sub{$Mojo::Weixin::Controller::VERSION};
 
@@ -171,18 +172,25 @@ sub clean_pid {
 
 sub kill_process {
     my $self = shift;
+    my $ret = 0;
     if(!$_[0] or $_[0]!~/^\d+$/){
         $self->error("pid无效，无法终止进程");
         return;
     }
-    #if($^O  eq "MSWin32"){
+    if($^O  eq "MSWin32"){
     #    my $exitcode = 0;
     #    Win32::Process::KillProcess($_[0],$exitcode);
     #    return $exitcode;
-    #}
-    #else{ 
-        kill POSIX::SIGINT,$_[0] ;
-    #}
+        $ret = kill POSIX::SIGINT,$_[0] ;
+    }
+    else{ 
+        $ret = kill POSIX::SIGTERM,$_[0] ;
+    }
+    
+    #client进程退出没有那么快，马上检查的话，仍然是存在的，干脆先不检查了
+    #return !$self->check_process($_[0]);
+
+    return $ret;
 }
 sub check_process {
     my $self = shift;
@@ -202,6 +210,9 @@ sub start_client {
     if(!$param->{client}){
         return {code => 1, status=>'client not found',};
     }
+    elsif($self->max_client < keys %{$self->backend}){
+        return {code => 5, status=>'max clients exceed'};
+    }
     elsif(exists $self->backend->{$param->{client}}){
         if( $self->check_process($self->backend->{$param->{client}}{pid}) ){
             my %client = %{ $self->backend->{$param->{client}} };
@@ -242,6 +253,7 @@ sub start_client {
     $ENV{MOJO_WEIXIN_DISABLE_COLOR} = $self->disable_color;
     $ENV{MOJO_WEIXIN_HTTP_DEBUG} = $self->http_debug;
     $ENV{MOJO_WEIXIN_LOG_LEVEL} = $self->log_level;
+    $ENV{MOJO_WEIXIN_CONTROLLER_PID} = $$;
 
     $ENV{MOJO_WEIXIN_TMPDIR} = $self->tmpdir if not defined $ENV{MOJO_WEIXIN_TMPDIR};
     $ENV{MOJO_WEIXIN_STATE_PATH} = File::Spec->catfile($ENV{MOJO_WEIXIN_TMPDIR},join('','mojo_weixin_state_',$ENV{MOJO_WEIXIN_ACCOUNT},'.json')) if not defined $ENV{MOJO_WEIXIN_STATE_PATH};
@@ -252,9 +264,9 @@ sub start_client {
         my $template =<<'MOJO_WEIXIN_CLIENT_TEMPLATE';
 #!/usr/bin/env perl
 use Mojo::Weixin;
+$|=1;
 my $client = Mojo::Weixin->new(log_head=>"[$ENV{MOJO_WEIXIN_ACCOUNT}][$$]");
 $0 = "wxclient(" . $client->account . ")" if $^O ne "MSWin32";
-$SIG{INT} = 'IGNORE' if ($^O ne 'MSWin32' and !-t);
 $client->load(["ShowMsg","UploadQRcode"]);
 $client->load("Openwx",data=>{listen=>[{host=>"127.0.0.1",port=>$ENV{MOJO_WEIXIN_PLUGIN_OPENWX_PORT} }], post_api=>$ENV{MOJO_WEIXIN_PLUGIN_OPENWX_POST_API} || undef,post_event=>$ENV{MOJO_WEIXIN_PLUGIN_OPENWX_POST_EVENT} // 1,post_media_data=> $ENV{MOJO_WEIXIN_PLUGIN_OPENWX_POST_MEDIA_DATA} // 1, poll_api=>$ENV{MOJO_WEIXIN_PLUGIN_OPENWX_POLL_API} || undef, poll_interval => $ENV{MOJO_WEIXIN_PLUGIN_OPENWX_POLL_INTERVAL} },call_on_load=>1);
 $client->run();
