@@ -16,11 +16,20 @@ sub call{
     my $data = shift;
     $client->die("请先安装模块 Mojo::IRC::Server::Chinese") if not $Mojo::Weixin::Plugin::IRCShell::has_mojo_irc_server;
     my $master_irc_nick = $data->{master_irc_nick};
-    my $upload_api = $data->{upload_api} // 'https://sm.ms/api/upload';
-    my $is_load_friend = defined $data->{load_friend}?$data->{load_friend}:1;
-    my @groups = ref($data->{group}) eq "ARRAY"?@{$data->{group}}:();
+    my $upload_api = $data->{upload_api}; #'http://img.vim-cn.com/';
+    my $is_load_friend = defined $data->{load_friend}?$data->{load_friend}:0;
     my %mode = ref($data->{mode}) eq "HASH"?%{$data->{mode}}:();
-    $ircd = Mojo::IRC::Server::Chinese->new(listen=>$data->{listen},log=>$client->log);
+    $data->{auto_join_channel} = 1 if not defined $data->{auto_join_channel};
+    $data->{create_chanserv_user} = 1 if not defined $data->{create_chanserv_user};
+    $ircd = Mojo::IRC::Server::Chinese->new(listen=>$data->{listen},log=>$client->log,auth=>$data->{auth});
+    if($data->{create_chanserv_user}){
+        my $chanserv= $ircd->new_user(id=>"__ChanServ__",name=>"ChanServ:虚拟用户",user=>"__ChanServ__",nick=>"ChanServ",virtual=>1);
+        $ircd->on(new_channel=>sub{
+            my ($ircd,$channel) = @_;
+            return if index( $channel->mode ,"v" ) == -1;
+            $chanserv->join_channel($channel) if not $chanserv->is_join_channel($channel);
+        });
+    }
     $ircd->on(privmsg=>sub{
         my($ircd,$user,$msg) = @_;
         if(substr($msg->{params}[0],0,1) eq "#" ){
@@ -37,23 +46,36 @@ sub call{
                 $raw_content = $content;
             }
             if($user->nick ne $master_irc_nick and !$user->is_localhost){
-                #$content = $user->nick . ": $content"; 
+                #$content = $user->nick . ": $content";
                 $content .= "\n(来自irc用户 - ".$user->nick.")";
             }
-            $group->send($content,sub{
+
+            # 群组会话里面发送文件的二级命令解析 i.e. sendFile /tmp/dog.png
+            if ($content =~ m/^(?:sendFile |!!)([^\s]+)$/i) {
+                my $media_path = $1;
+                if(-f $media_path or $media_path=~/^https?:\/\//){
+                    $group->send_media($media_path);
+                }
+                else{
+                    $user->send($user->ident,"PRIVMSG",$channel->name,"[系统提示]无法发送，无效媒体：$media_path");
+                }
+            } 
+            else {
+              $group->send($content,sub{
                 $_[1]->from("irc");
                 $_[1]->cb(sub{
-                    my($client,$msg)=@_;
-                    if($msg->is_success){
-                        if($msg->content ne $raw_content){
-                            $msg->content($raw_content);
-                        }
-                    }
-                    else{
-                        $user->send($user->ident,"PRIVMSG",$channel_name,$content . "[发送失败]");
+                  my($client,$msg)=@_;
+                  if($msg->is_success){
+                    if($msg->content ne $raw_content){
+                      $msg->content($raw_content);
                     }
+                  }
+                  else{
+                    $user->send($user->ident,"PRIVMSG",$channel_name,$content . "[发送失败]");
+                  }
                 });
-            });
+              });
+            }
         }
         elsif($user->nick eq $master_irc_nick or $user->is_localhost){
             my $nick =  $msg->{params}[0];
@@ -63,7 +85,7 @@ sub call{
             if(not defined $u){
                 my $friend = $client->search_friend(displayname=>$nick);
                 return if not defined $friend;
-                my $channel = $ircd->search_channel(name=>'#我的好友') || $ircd->new_channel(name=>'#我的好友',mode=>"Pis");
+                my $channel = $ircd->search_channel(name=>'#我的好友') || $ircd->new_channel(name=>'#我的好友',mode=>"Pivs");
                 return if not defined $channel;
                 $u = $ircd->new_user(
                     id      =>$friend->id,
@@ -74,30 +96,43 @@ sub call{
                 );
                 $u->join_channel($channel);
                 $user->send($user->ident,"PRIVMSG",$nick,"[系统提示]已从微信好友中搜索到对应昵称好友并生成irc用户，现在可以继续和好友聊天了");
-            
+
             }
             my $friend = $client->search_friend(id=>$u->id);
             if(defined $friend){
-                $friend->send($content,sub{
+                # 好友会话里面发送文件二级命令解析 sendFile /tmp/dog.png
+                if ($content =~ m/^(?:sendFile |!!)([^\s]+)$/i) {
+                    my $media_path = $1;
+                    if(-f $media_path or $media_path=~/^https?:\/\//){
+                        $friend->send_media($media_path);
+                    }
+                    else{
+                        $user->send($user->ident,"PRIVMSG",$nick,"[系统提示]无法发送，无效媒体：$media_path");
+                    }
+                } 
+                else {
+                  $friend->send($content,sub{
                     $_[1]->from("irc");
                     $_[1]->cb(sub{
-                        my($client,$msg)=@_;
-                        return if $msg->is_success;
-                        $user->send($user->ident,"PRIVMSG",$nick,$content . "[发送失败]");
+                      my($client,$msg)=@_;
+                      return if $msg->is_success;
+                      $user->send($user->ident,"PRIVMSG",$nick,$content . "[发送失败]");
                     });
-                });
+                  });
+                }
             }
             else{
                 $user->send($user->ident,"PRIVMSG",$nick,$content . "[发送失败]");
                 $user->send($user->ident,"PRIVMSG",$nick,"你和 [$nick] 并非好友关系");
-            } 
+            }
         }
     });
 
     my $callback = sub{
         my %delete_channel  = map {$_->id => $_} grep {$_->name ne "#我的好友"}  $ircd->channels;
         $ircd->remove_user($_) for grep {$_->is_virtual} $ircd->users;
-        my $friend_channel = $ircd->new_channel(name=>'#我的好友',mode=>"Pis");
+        #$ircd->remove_user($_) for grep {$_->id ne '__ChanServ__' and $_->is_virtual} $ircd->users;
+        my $friend_channel = $ircd->new_channel(name=>'#我的好友',mode=>"Pivs");
         if($is_load_friend){
             $client->each_friend(sub{
                 my($client,$friend) = @_;
@@ -128,8 +163,10 @@ sub call{
         }
         $client->each_group(sub{
             my($client,$group) = @_;
-            return if(@groups and not first {$group->displayname eq $_} @groups);
-            my $mode = defined $mode{$group->displayname}?$mode{$group->displayname}:"Pi";
+            return if ref $data->{ban_group}  eq "ARRAY" and first {$group->displayname eq $_} @{$data->{ban_group}};
+            return if ref $data->{allow_group}  eq "ARRAY" and !first {$group->displayname eq $_} @{$data->{allow_group}};
+            my $mode = defined $mode{$group->displayname}?$mode{$group->displayname}:"Piv";
+            $mode .= "v" if index($mode,"v")==-1;
             my $channel_name = '#'.$group->displayname;$channel_name=~s/\s|,|&//g;
             my $channel = $ircd->search_channel(name=>$channel_name);
             if(defined $channel){
@@ -143,8 +180,10 @@ sub call{
     };
     $client->on(new_group=>sub{
         my($client,$group) = @_;
-        return if(@groups and not first {$group->displayname eq $_} @groups);
-        my $mode = defined $mode{$group->displayname}?$mode{$group->displayname}:"Pi";
+        return if ref $data->{ban_group}  eq "ARRAY" and first {$group->displayname eq $_} @{$data->{ban_group}};
+        return if ref $data->{allow_group}  eq "ARRAY" and !first {$group->displayname eq $_} @{$data->{allow_group}};
+        my $mode = defined $mode{$group->displayname}?$mode{$group->displayname}:"Piv";
+        $mode .= "v" if index($mode,"v")==-1;
         my $channel_name = '#'.$group->displayname;$channel_name=~s/\s|,|&//g;
         my $channel = $ircd->search_channel(name=>$channel_name);
         if(defined $channel){
@@ -155,8 +194,8 @@ sub call{
     });
     $client->on(lose_group=>sub{
         my($client,$group) = @_;
-        return if(@groups and not first {$group->displayname eq $_} @groups);
-        my $mode = defined $mode{$group->displayname}?$mode{$group->displayname}:"Pi";
+        return if ref $data->{ban_group}  eq "ARRAY" and first {$group->displayname eq $_} @{$data->{ban_group}};
+        return if ref $data->{allow_group}  eq "ARRAY" and !first {$group->displayname eq $_} @{$data->{allow_group}};
         my $channel_name = '#'.$group->displayname;$channel_name=~s/\s|,|&//g;
         my $channel = $ircd->search_channel(name=>$channel_name);
         $ircd->remove_channel($channel) if defined $channel;
@@ -171,7 +210,7 @@ sub call{
         if($msg->type eq "friend_message"){
             my $friend = $msg->sender;
             my $user = $ircd->search_user(id=>$friend->id,virtual=>1) || $ircd->search_user(nick=>$friend->displayname,virtual=>0);
-            my $channel = $ircd->search_channel(name=>'#我的好友') || $ircd->new_channel(name=>'#我的好友',mode=>"Pis");
+            my $channel = $ircd->search_channel(name=>'#我的好友') || $ircd->new_channel(name=>'#我的好友',mode=>"Pivs");
             return if not defined $channel;
             if(not defined $user){
                 $user = $ircd->new_user(
@@ -190,15 +229,12 @@ sub call{
             if(defined $upload_api and $msg->format eq 'media'){
                 my $content = $msg->content;
                 #接收的图片上传到图床
-                $client->http_post($upload_api,{json=>1},form=>{format=>'json',smfile=>{content=>$msg->media_data}},sub{
-                    my($json,$ua,$tx)=@_;
-                    if(not defined $json){
+                $client->http_post($upload_api,form=>{image=>{content=>$msg->media_data}},sub{
+                    my($url,$ua,$tx)=@_;
+                    if(not defined $url or $url!~/https?:\/\//){
                         $client->warn("二维码图片上传云存储失败: 响应数据异常");
                     }
-                    elsif(defined $json and $json->{code} ne 'success' ){
-                        $client->warn("二维码图片上传云存储失败: " . $json->{msg});
-                    }
-                    else{$content =~ s/^\[(.+?)\]\(.+?\)/[$1]($json->{data}{url})/g;}
+                    else{$content =~ s/^\[(.+?)\]\(.+?\)/[$1]($url)/g;}
                     for (grep { $_->nick eq $master_irc_nick or $_->is_localhost} grep {!$_->is_virtual} $ircd->users){
                         for my $line (split /\r?\n/,$content){
                             $_->send($user->ident,"PRIVMSG",$_->nick,$line);
@@ -220,7 +256,8 @@ sub call{
         elsif($msg->type eq "group_message"){
             my $member = $msg->sender;
             #return if not defined $member;
-            return if @groups and not first {$member->group->displayname eq $_} @groups;
+            return if ref $data->{ban_group}  eq "ARRAY" and first {$msg->group->displayname eq $_} @{$data->{ban_group}};
+            return if ref $data->{allow_group}  eq "ARRAY" and !first {$msg->group->displayname eq $_} @{$data->{allow_group}};
             my $user = $ircd->search_user(id=>$member->id,virtual=>1) || $ircd->search_user(nick=>$member->displayname,virtual=>0);
             my $channel = $ircd->search_channel(id=>$member->group->id) ||
                     $ircd->new_channel(id=>$member->group->id,name=>'#'.$member->group->displayname,);
@@ -233,7 +270,7 @@ sub call{
                     nick    =>$member->displayname,
                     virtual => 1,
                 );
-                
+
                 $user->join_channel($channel);
             }
             elsif($user->is_virtual){
@@ -243,19 +280,23 @@ sub call{
                 $user->join_channel($channel) if not $user->is_join_channel($channel);
             }
 
-            
+            #master用户如果没有加入到该频道就自动加入，防止漏收消息
+            if($data->{auto_join_channel}){
+                for(grep {$_->nick eq $master_irc_nick or $_->is_localhost}
+                    grep {!$_->is_virtual} $ircd->users){
+                    $_->join_channel($channel) if not $_->is_join_channel($channel);
+                }
+            }
+
             if(defined $upload_api and $msg->format eq 'media'){
                 my $content = $msg->content;
                 #接收的图片上传到图床
-                $client->http_post($upload_api,{json=>1},form=>{format=>'json',smfile=>{content=>$msg->media_data}},sub{
-                    my($json,$ua,$tx)=@_;
-                    if(not defined $json){
+                $client->http_post($upload_api,form=>{image=>{content=>$msg->media_data}},sub{
+                    my($url,$ua,$tx)=@_;
+                    if(not defined $url or $url!~/https?:\/\//){
                         $client->warn("二维码图片上传云存储失败: 响应数据异常");
                     }
-                    elsif(defined $json and $json->{code} ne 'success' ){
-                        $client->warn("二维码图片上传云存储失败: " . $json->{msg});
-                    }
-                    else{$content =~ s/^\[(.+?)\]\(.+?\)/[$1]($json->{data}{url})/g;}
+                    else{$content =~ s/^\[(.+?)\]\(.+?\)/[$1]($url)/g;}
                     for(grep {!$_->is_virtual} $channel->users){
                         my @content = split /\r?\n/,$content;
                         if($content[0]=~/^\@([^\s]+?) /){
@@ -273,7 +314,7 @@ sub call{
                 });
             }
             else{
-                for(grep {!$_->is_virtual} $channel->users){
+                for(grep {!$_->is_virtual} ($channel->users)){
                     my @content = split /\r?\n/,$msg->content;
                     if($content[0]=~/^\@([^\s]+?) /){
                         my $at_nick = $1;
@@ -288,7 +329,7 @@ sub call{
                 }
             }
         }
-    
+
     });
     $client->on(send_message=>sub{
         my($client,$msg) = @_;
@@ -296,7 +337,7 @@ sub call{
         if($msg->type eq "friend_message"){
             my $friend = $msg->receiver;
             my $user = $ircd->search_user(id=>$friend->id,virtual=>1) || $ircd->search_user(nick=>$friend->displayname,virtual=>0);
-            my $channel = $ircd->search_channel(name=>'#我的好友') || $ircd->new_channel(name=>'#我的好友',mode=>"Pis");
+            my $channel = $ircd->search_channel(name=>'#我的好友') || $ircd->new_channel(name=>'#我的好友',mode=>"Pivs");
             if(not defined $user){
                 $user=$ircd->new_user(
                     id      =>$friend->id,
@@ -310,18 +351,16 @@ sub call{
             elsif($user->is_virtual){
                 $user->join_channel($channel)  if not $user->is_join_channel($channel);
             }
+
             if(defined $upload_api and $msg->format eq 'media'){
                 my $content = $msg->content;
                 #发送的图片上传到图床
-                $client->http_post($upload_api,{json=>1},form=>{format=>'json',smfile=>{content=>$msg->media_data}},sub{
-                    my($json,$ua,$tx)=@_;
-                    if(not defined $json){
+                $client->http_post($upload_api,form=>{image=>{content=>$msg->media_data}},sub{
+                    my($url,$ua,$tx)=@_;
+                    if(not defined $url or $url!~/https?:\/\//){
                         $client->warn("二维码图片上传云存储失败: 响应数据异常");
                     }
-                    elsif(defined $json and $json->{code} ne 'success' ){
-                        $client->warn("二维码图片上传云存储失败: " . $json->{msg});
-                    }   
-                    else{$content =~ s/^\[(.+?)\]\(.+?\)/[$1]($json->{data}{url})/g;}
+                    else{$content =~ s/^\[(.+?)\]\(.+?\)/[$1]($url)/g;}
                     for(
                         grep {$_->nick eq $master_irc_nick or $_->is_localhost}
                         grep {!$_->is_virtual} $ircd->users
@@ -336,7 +375,7 @@ sub call{
             }
             else{
                 for(
-                    grep {$_->nick eq $master_irc_nick or $_->is_localhost} 
+                    grep {$_->nick eq $master_irc_nick or $_->is_localhost}
                     grep {!$_->is_virtual} $ircd->users
                 )
                 {
@@ -348,20 +387,27 @@ sub call{
             }
         }
         elsif($msg->type eq "group_message"){
-            return if @groups and not first {$msg->group->displayname eq $_} @groups;
+            return if ref $data->{ban_group}  eq "ARRAY" and first {$msg->group->displayname eq $_} @{$data->{ban_group}};
+            return if ref $data->{allow_group}  eq "ARRAY" and !first {$msg->group->displayname eq $_} @{$data->{allow_group}};
             my $channel = $ircd->search_channel(id=>$msg->group->id);
             return unless defined $channel;
+
+            #master用户如果没有加入到该频道就自动加入，防止漏收消息
+            if($data->{auto_join_channel}){
+                for(grep {$_->nick eq $master_irc_nick or $_->is_localhost}
+                    grep {!$_->is_virtual} $ircd->users){
+                    $_->join_channel($channel) if not $_->is_join_channel($channel);
+                }
+            }
+
             if(defined $upload_api and $msg->format eq 'media'){
                 my $content = $msg->content;
-                $client->http_post($upload_api,{json=>1},form=>{format=>'json',smfile=>{content=>$msg->media_data}},sub{
-                    my($json,$ua,$tx)=@_;
-                    if(not defined $json){
+                $client->http_post($upload_api,form=>{image=>{content=>$msg->media_data}},sub{
+                    my($url,$ua,$tx)=@_;
+                    if(not defined $url or $url!~/https?:\/\//){
                         $client->warn("二维码图片上传云存储失败: 响应数据异常");
                     }
-                    elsif(defined $json and $json->{code} ne 'success' ){
-                        $client->warn("二维码图片上传云存储失败: " . $json->{msg});
-                    }
-                    else{$content =~ s/^\[(.+?)\]\(.+?\)/[$1]($json->{data}{url})/g;}
+                    else{$content =~ s/^\[(.+?)\]\(.+?\)/[$1]($url)/g;}
                     for my $master_irc_client (
                         grep {$_->nick eq $master_irc_nick or $_->is_localhost}
                         grep {!$_->is_virtual} $ircd->users
@@ -411,13 +457,17 @@ sub call{
             return if $property ne "name" and $property ne "markname";
             my $user = $ircd->search_user(id=>$object->id,virtual=>1);
             return unless defined $user;
-            $user->set_nick($object->displayname) if $object->displayname ne $user->nick;
+            my $displayname = $object->displayname;
+            $displayname=~s/\s|\@|!//g;
+            $user->set_nick($displayname) if $displayname ne $user->nick;
         }
         elsif($object->is_group_member){
-            return if $property ne "name" and $property ne "markname"; 
+            return if $property ne "name" and $property ne "markname";
             my $user = $ircd->search_user(id=>$object->id,virtual=>1);
             return unless defined $user;
-            $user->set_nick($object->displayname) if $object->displayname ne $user->nick;
+            my $displayname = $object->displayname;
+            $displayname=~s/\s|\@|!//g;
+            $user->set_nick($displayname) if $displayname ne $user->nick;
         }
         elsif($object->is_group){
             return if $property ne "name";
